﻿' ================================================================================
' Developed by Black Falcon Software, Inc.
' Copyright 2017
' License - MIT (Open Source)
' 
' Restrictions: Module cannot be sold for profit in "As Is" state
'               Module can only be sold commercially as part of a larger product
' ================================================================================


Imports System
Imports System.Data
Imports System.Collections

Imports FirebirdSql.Data.FirebirdClient


Public Class FbSqlHelper

    #Region "Private Class Level Variables"
    
        Private ciTimeOut  As Integer = 60

    #End Region


    #Region "Constructor(s)"

        Public Sub New()

        End Sub

    #End Region


    #Region "Public Methods - Test Database Connection"

        Public Function Test_DatabaseConnection(ByVal psConnectionString As String) As Integer

            Dim loFbSqlConnection  As FbConnection = Nothing

            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try

            Return (0)
        End Function

    #End Region


    #Region "Public Methods - ExecuteDataReader"
    
        Public Overloads Function ExecuteDataReader(ByVal psConnectionString As String, _
                                                    ByVal psQuery As String) As FbDataReader
      
            Dim loFbSqlDataReader  As FbDataReader = Nothing
            Dim loFbSqlConnection  As FbConnection = Nothing
            Dim loFbSqlCommand     As FbCommand = Nothing

        
            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

                    If (psQuery.ToUpper().IndexOf("SELECT ") > -1) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    
          
                loFbSqlDataReader = loFbSqlCommand.ExecuteReader(CommandBehavior.CloseConnection)
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            End Try
        
            Return (loFbSqlDataReader)
        End Function


        Public Overloads Function ExecuteDataReader(ByVal psConnectionString As String, _
                                                    ByVal psQuery As String, _
                                                    ByRef poParameterStructures As ArrayList) As FbDataReader

            Dim loDataSet          As DataSet = Nothing
            Dim loFbSqlDataReader  As FbDataReader '= Nothing
            Dim loFbSqlConnection  As FbConnection = Nothing
            Dim loFbSqlParameter   As FbParameter = Nothing
            Dim loFbSqlCommand     As FbCommand = Nothing


            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)
          
                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut
          

                    ' check for parameterized inline SQL or stored-procedure
                    If ((psQuery.IndexOf("@") > -1) Or _
                        (psQuery.ToUpper().IndexOf("SELECT ") > -1)) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    
    

                    ' create parameters for stored-procedure
                    If (Not poParameterStructures Is Nothing) Then
                        For Each loFbSqlHelperParameterStructure As FbSqlHelperParameterStructure In poParameterStructures
                            loFbSqlParameter = New FbParameter() 

                                loFbSqlParameter.ParameterName = loFbSqlHelperParameterStructure.PARAMETER_NAME
                                loFbSqlParameter.FbDbType = loFbSqlHelperParameterStructure.PARAMETER_FBSQLDBTYPE
                                loFbSqlParameter.Direction = loFbSqlHelperParameterStructure.PARAMETER_DIRECTION

                                Select Case loFbSqlParameter.Direction  
                                    Case ParameterDirection.Input:
                                        loFbSqlParameter.Value = loFbSqlHelperParameterStructure.PARAMETER_VALUE
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE

                                    Case ParameterDirection.Output:
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE
                                End Select

                            loFbSqlCommand.Parameters.Add(loFbSqlParameter)
                        Next
                    End If

                loFbSqlDataReader = loFbSqlCommand.ExecuteReader(CommandBehavior.CloseConnection)
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            'Finally
            '    loFbSqlCommand.Dispose()
            End Try

            Return (loFbSqlDataReader)
        End Function
  
    #End Region


    #Region "Public Methods - ExecuteDataSet"

        Public Overloads Function ExecuteDataSet(ByVal psConnectionString As String, _
                                                 ByVal psQuery As String) As DataSet
      
            Dim loDataSet           As DataSet = Nothing
            Dim loFbSqlConnection   As FbConnection = Nothing
            Dim loFbSqlCommand      As FbCommand = Nothing
            Dim loFbSqlDataAdapter  As FbDataAdapter = Nothing

        
            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

                    If (psQuery.ToUpper().IndexOf("SELECT ") > -1) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    
          
                    loFbSqlDataAdapter = New FbDataAdapter
                    loFbSqlDataAdapter.SelectCommand = loFbSqlCommand
                    loDataSet = New DataSet

                loFbSqlDataAdapter.Fill(loDataSet, "DATA")
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlCommand.Dispose()
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try
        
            Return (loDataSet)
        End Function


        Public Overloads Function ExecuteDataSet(ByVal psConnectionString As String, _
                                                 ByVal psQuery As String, _
                                                 ByRef poParameterStructures As ArrayList) As DataSet

            Dim loParameterStructures  As New ArrayList()

            Dim loDataSet                As DataSet
            Dim loFbSqlConnection        As FbConnection = Nothing
            Dim loFbSqlParameter         As FbParameter
            Dim loFbSqlCommand           As FbCommand = Nothing
            Dim loFbSqlDataAdapter       As FbDataAdapter = Nothing


            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut
          
                    ' check for parameterized inline SQL or stored-procedure
                    If ((psQuery.IndexOf("@") > -1) Or _
                        (psQuery.ToUpper().IndexOf("SELECT ") > -1)) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else        
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If  
    

                    ' create parameters for stored-procedure
                    If (Not poParameterStructures Is Nothing) Then
                        For Each loFbSqlHelperParameterStructure As FbSqlHelperParameterStructure In poParameterStructures
                            loFbSqlParameter = New FbParameter() 

                                loFbSqlParameter.ParameterName = loFbSqlHelperParameterStructure.PARAMETER_NAME
                                loFbSqlParameter.FbDbType = loFbSqlHelperParameterStructure.PARAMETER_FBSQLDBTYPE
                                loFbSqlParameter.Direction = loFbSqlHelperParameterStructure.PARAMETER_DIRECTION

                                Select Case loFbSqlParameter.Direction  
                                    Case ParameterDirection.Input:
                                        loFbSqlParameter.Value = loFbSqlHelperParameterStructure.PARAMETER_VALUE
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE

                                    Case ParameterDirection.Output:
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE
                                End Select

                            loFbSqlCommand.Parameters.Add(loFbSqlParameter)
                        Next
                    End If

                loFbSqlDataAdapter = New FbDataAdapter()                
                loFbSqlDataAdapter.SelectCommand = loFbSqlCommand

                loDataSet = New DataSet()
                loFbSqlDataAdapter.Fill(loDataSet)
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                'loFbSqlCommand.Dispose()   DO NOT Dispose loFbSqlCommand as it causes the reference pointers into the dataset to be flushed
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try
        

            poParameterStructures = loParameterStructures

            Return (loDataSet)
        End Function

    #End Region


    #Region "Public Methods - ExecuteXml"

        Public Overloads Function ExecuteXml(ByVal psConnectionString As String, _
                                             ByVal psQuery As String) As Object
      
            Dim loXMLObject         As Object = ""

            Dim loDataSet           As DataSet = Nothing
            Dim loFbSqlConnection   As FbConnection = Nothing
            Dim loFbSqlCommand      As FbCommand = Nothing
            Dim loFbSqlDataAdapter  As FbDataAdapter = Nothing

        
            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

                    If (psQuery.ToUpper().IndexOf("SELECT ") > -1) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    

                    loFbSqlDataAdapter = New FbDataAdapter
                    loFbSqlDataAdapter.SelectCommand = loFbSqlCommand
                    loDataSet = New DataSet

                    loFbSqlDataAdapter.Fill(loDataSet, "DATA")

                loXMLObject = loDataSet.GetXml
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                'loFbSqlCommand.Dispose()   DO NOT Dispose loFbSqlCommand as it causes the reference pointers into the dataset to be flushed
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try
        
            Return (loXMLObject)
        End Function


        Public Overloads Function ExecuteXml(ByVal psConnectionString As String, _
                                             ByVal psQuery As String, _
                                             ByRef poParameterStructures As ArrayList) As Object

            Dim loParameterStructures  As New ArrayList()

            Dim loXMLObject            As Object = ""

            Dim loDataSet              As DataSet = Nothing
            Dim loFbSqlConnection      As FbConnection = Nothing
            Dim loFbSqlParameter       As FbParameter
            Dim loFbSqlCommand         As FbCommand = Nothing
            Dim loFbSqlDataAdapter     As FbDataAdapter = Nothing


            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)
          
                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

          
                    ' check for parameterized inline SQL or stored-procedure
                    If ((psQuery.IndexOf("@") > -1) Or _
                        (psQuery.ToUpper().IndexOf("SELECT ") > -1)) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else        
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If  
    

                    ' create parameters for stored-procedure
                    If (Not poParameterStructures Is Nothing) Then
                        For Each loFbSqlHelperParameterStructure As FbSqlHelperParameterStructure In poParameterStructures
                            loFbSqlParameter = New FbParameter() 

                                loFbSqlParameter.ParameterName = loFbSqlHelperParameterStructure.PARAMETER_NAME
                                loFbSqlParameter.FbDbType = loFbSqlHelperParameterStructure.PARAMETER_FBSQLDBTYPE
                                loFbSqlParameter.Direction = loFbSqlHelperParameterStructure.PARAMETER_DIRECTION

                                Select Case loFbSqlParameter.Direction  
                                    Case ParameterDirection.Input:
                                        loFbSqlParameter.Value = loFbSqlHelperParameterStructure.PARAMETER_VALUE
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE

                                    Case ParameterDirection.Output:
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE
                                End Select

                            loFbSqlCommand.Parameters.Add(loFbSqlParameter)
                        Next
                    End If

                    loFbSqlDataAdapter = New FbDataAdapter
                    loFbSqlDataAdapter.SelectCommand = loFbSqlCommand
                    loDataSet = New DataSet

                loFbSqlDataAdapter.Fill(loDataSet, "DATA")

                loXMLObject = loDataSet.GetXml
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                'loFbSqlCommand.Dispose()   DO NOT Dispose loFbSqlCommand as it causes the reference pointers into the dataset to be flushed
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try
        

            poParameterStructures = loParameterStructures

            Return (loXMLObject)
        End Function

    #End Region


    #Region "Public Methods - ExecuteScalar"

        Public Overloads Function ExecuteScalar(ByVal psConnectionString As String, _
                                                ByVal psQuery As String) As Object

            Dim loScalarObject     As Object = ""

            Dim loFbSqlConnection  As FbConnection = Nothing
            Dim loFbSqlCommand     As FbCommand = Nothing

        
            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

                    If (psQuery.ToUpper().IndexOf("SELECT ") > -1) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    
          
                loScalarObject = loFbSqlCommand.ExecuteScalar()
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlCommand.Dispose()
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try
        
            Return (loScalarObject)
        End Function


        Public Overloads Function ExecuteScalar(ByVal psConnectionString As String, _
                                                ByVal psQuery As String, _
                                                ByRef poParameterStructures As ArrayList) As Object

            Dim loParameterStructures  As New ArrayList()

            Dim loScalarObject         As Object = ""

            Dim loDataSet              As DataSet = Nothing
            Dim loFbSqlConnection      As FbConnection = Nothing
            Dim loFbSqlParameter       As FbParameter
            Dim loFbSqlCommand         As FbCommand = Nothing
            Dim loFbSqlDataAdapter     As FbDataAdapter = Nothing


            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)
          
                    loFbSqlCommand = New FbCommand(psQuery, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

          
                    ' check for parameterized inline SQL or stored-procedure
                    If ((psQuery.IndexOf("@") > -1) Or _
                        (psQuery.ToUpper().IndexOf("SELECT ") > -1)) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else        
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If  
    

                    ' create parameters for stored-procedure
                    If (Not poParameterStructures Is Nothing) Then
                        For Each loFbSqlHelperParameterStructure As FbSqlHelperParameterStructure In poParameterStructures
                            loFbSqlParameter = New FbParameter() 

                                loFbSqlParameter.ParameterName = loFbSqlHelperParameterStructure.PARAMETER_NAME
                                loFbSqlParameter.FbDbType = loFbSqlHelperParameterStructure.PARAMETER_FBSQLDBTYPE
                                loFbSqlParameter.Direction = loFbSqlHelperParameterStructure.PARAMETER_DIRECTION

                                Select Case loFbSqlParameter.Direction  
                                    Case ParameterDirection.Input:
                                        loFbSqlParameter.Value = loFbSqlHelperParameterStructure.PARAMETER_VALUE
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE

                                    Case ParameterDirection.Output:
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE
                                End Select

                            loFbSqlCommand.Parameters.Add(loFbSqlParameter)
                        Next
                    End If

                loScalarObject = loFbSqlCommand.ExecuteScalar()


                'update any parameter structure values from output parameters
                'For Each loMySqlHelperParameterStructure As MySqlHelperParameterStructure In poParameterStructures
                '    If ((loMySqlHelperParameterStructure.PARAMETER_DIRECTION = ParameterDirection.InputOutput) Or _
                '        (loMySqlHelperParameterStructure.PARAMETER_DIRECTION = ParameterDirection.Output)) Then

                '        For Each loMySqlParameter In loMySqlCommand.Parameters
                '            If (loMySqlParameter.ParameterName.Trim() = loMySqlHelperParameterStructure.PARAMETER_NAME.Trim())
                '                loMySqlHelperParameterStructure.PARAMETER_VALUE = loMySqlParameter.Value.ToString().Trim()
                '            End If
                '        Next
                '    End If

                '    If (Not loParameterStructures.Contains(loMySqlHelperParameterStructure)) Then
                '        loParameterStructures.Add(loMySqlHelperParameterStructure)
                '    End If
                'Next
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlCommand.Dispose()
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try


            poParameterStructures = loParameterStructures
        
            Return (loScalarObject)
        End Function

    #End Region


    #Region "Public Methods - ExecuteNonQuery"

        Public Overloads Function ExecuteNonQuery(ByVal psConnectionString As String, _
                                                  ByVal psAction As String) As Integer

            Dim liRecordsAffected  As Integer = 0
            Dim loFbSqlConnection  As FbConnection = Nothing
            Dim loFbSqlCommand     As FbCommand = Nothing

        
            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psAction, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

                    If ((psAction.ToUpper().IndexOf("DELETE ") > -1) Or _
                        (psAction.ToUpper().IndexOf("INSERT ") > -1) Or _
                        (psAction.ToUpper().IndexOf("UPDATE ") > -1)) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    
          
                liRecordsAffected = loFbSqlCommand.ExecuteNonQuery
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlCommand.Dispose()
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try
        
            Return (liRecordsAffected)
        End Function


        Public Overloads Function ExecuteNonQuery(ByVal psConnectionString As String, _
                                                  ByVal psAction As String, _
                                                  ByRef poParameterStructures As ArrayList) As Integer

            Dim liRecordsAffected      As Integer = 0

            Dim loParameterStructures  As New ArrayList()

            Dim loFbSqlConnection      As FbConnection = Nothing
            Dim loFbSqlCommand         As FbCommand = Nothing
            Dim loFbSqlParameter       As FbParameter

        
            Try 
                loFbSqlConnection = Get_DBConnection(psConnectionString)

                    loFbSqlCommand = New FbCommand(psAction, loFbSqlConnection)
                    loFbSqlCommand.CommandTimeout = ciTimeOut

                    If ((psAction.ToUpper().IndexOf("DELETE ") > -1) Or _
                        (psAction.ToUpper().IndexOf("INSERT ") > -1) Or _
                        (psAction.ToUpper().IndexOf("UPDATE ") > -1)) Then
                        loFbSqlCommand.CommandType = CommandType.Text
                    Else
                        loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                    End If    


                    ' create parameters for stored-procedure
                    If (Not poParameterStructures Is Nothing) Then
                        For Each loFbSqlHelperParameterStructure As FbSqlHelperParameterStructure In poParameterStructures
                            loFbSqlParameter = New FbParameter() 

                                loFbSqlParameter.ParameterName = loFbSqlHelperParameterStructure.PARAMETER_NAME
                                loFbSqlParameter.FbDbType = loFbSqlHelperParameterStructure.PARAMETER_FBSQLDBTYPE
                                loFbSqlParameter.Direction = loFbSqlHelperParameterStructure.PARAMETER_DIRECTION

                                Select Case loFbSqlParameter.Direction  
                                    Case ParameterDirection.Input:
                                        loFbSqlParameter.Value = loFbSqlHelperParameterStructure.PARAMETER_VALUE
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE

                                    Case ParameterDirection.Output:
                                        loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE
                                End Select

                            loFbSqlCommand.Parameters.Add(loFbSqlParameter)
                        Next
                    End If

                liRecordsAffected = loFbSqlCommand.ExecuteNonQuery

      
                'update any parameter structure values from output parameters
                For Each loFbSqlHelperParameterStructure As FbSqlHelperParameterStructure In poParameterStructures
                    If ((loFbSqlHelperParameterStructure.PARAMETER_DIRECTION = ParameterDirection.InputOutput) Or _
                        (loFbSqlHelperParameterStructure.PARAMETER_DIRECTION = ParameterDirection.Output)) Then

                        For Each loFbSqlParameter In loFbSqlCommand.Parameters
                            If (loFbSqlParameter.ParameterName.Trim() = loFbSqlHelperParameterStructure.PARAMETER_NAME.Trim())
                                If (loFbSqlParameter.Value.GetType.Name.Equals("Byte[]")) Then
                                    loFbSqlHelperParameterStructure.PARAMETER_VALUE = System.Text.ASCIIEncoding.ASCII.GetString(loFbSqlParameter.Value).Trim()
                                Else
                                    loFbSqlHelperParameterStructure.PARAMETER_VALUE = loFbSqlParameter.Value.ToString().Trim()
                                End If
                                
                            End If
                        Next
                    End If

                    If (Not loParameterStructures.Contains(loFbSqlHelperParameterStructure)) Then
                        loParameterStructures.Add(loFbSqlHelperParameterStructure)
                    End If
                Next
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlCommand.Dispose()
                loFbSqlConnection.Close()
                loFbSqlConnection.Dispose()
            End Try


            poParameterStructures = loParameterStructures
        
            Return (liRecordsAffected)
        End Function

    #End Region


    #Region "Public Support Methods"

        Public Function Replace_QuotesWithDoubleQuotes(psData As String) As String
    
            Return (psData.Replace(Strings.Chr(39), Strings.Chr(34)))
        End Function


        Public Function Replace_DoubleQuotesWithQuotes(psData As String) As String
    
            Return (psData.Replace(Strings.Chr(34), Strings.Chr(39)))
        End Function

    #End Region


    #Region "Private Support Methods"
    
        Private Function Get_DBConnection(ByVal psConnectionString As String) As FbConnection

            Dim loFbSqlHelperCommon  As New FbSqlHelperCommon()


            Return (loFbSqlHelperCommon.Get_DBConnection(psConnectionString))
        End Function

    #End Region

End Class
