﻿' ================================================================================
' Developed by Black Falcon Software, Inc.
' Copyright 2017
' License - MIT (Open Source)
' 
' Restrictions: Module cannot be sold for profit in "As Is" state
'               Module can only be sold commercially as part of a larger product
' ================================================================================

Imports System
Imports System.Data
Imports System.Collections

Imports FirebirdSql.Data.FirebirdClient


Public Class FbSqlHelperTransactionControl

    #Region "Constructor(s)"

        Public Sub New()

        End Sub

    #End Region


    #Region "Public Methods - Transaction Support"

        Public Function Execute_TransactionUpdate(ByVal psConnectionString As String, _
                                                  ByVal psAction As String, _
                                                  ByRef poSQLParameters As ArrayList, _
                                                  ByRef poFbSqlConnection As FbConnection, _
                                                  ByRef poFbSqlTransaction As FbTransaction) As Integer
			
		    Dim liRecordsAffected  As Integer = 0

            Dim loFbSqlHelper      As New BlackFalconAB.Firebird.FbSqlHelper()


            If (poFbSqlTransaction Is Nothing) Then
			    poFbSqlConnection = GetFbSqlConnectionObject(psConnectionString)
			    poFbSqlTransaction = GetFbSqlTransactionObject(poFbSqlConnection)  
            End If


			Try
    			liRecordsAffected = ExecuteNonQuery(psAction, poSQLParameters, poFbSqlConnection, poFbSqlTransaction)
			Catch loException As Exception
			    Try 
				    Rollback_FbSqlTransaction(poFbSqlTransaction)
			    Catch loException1 As Exception
                    Throw loException1
			    End Try

			    Throw loException
			End Try  

            Return (liRecordsAffected)
		End Function 


        Public Function Execute_TransactionCommit(ByRef poFbSqlConnection As FbConnection, _
                                                  ByRef poFbSqlTransaction As FbTransaction) As Integer

            Try
                Commit_FbSqlTransaction(poFbSqlTransaction)

				poFbSqlConnection.Close()
                poFbSqlConnection.Dispose()
                poFbSqlTransaction.Dispose()
			Catch loException As Exception
                Throw loException
			End Try  

            Return (0)
        End Function

    #End Region


    #Region "Private Methods - ExecuteNonQuery"

        Private Function ExecuteNonQuery(ByVal psAction As String, _
                                         ByRef poParameterStructures As ArrayList, _
                                         ByRef poFbSqlConnection As FbConnection, _
                                         ByRef poFbSqlTransaction As FbTransaction) As Integer

            Dim liRecordsAffected                As Integer = 0

            Dim loParameterStructures            As New ArrayList()

            Dim loFbSqlCommand                   As FbCommand = Nothing
            Dim loFbSqlParameter                 As FbParameter
            Dim loFbSqlHelperParameterStructure  As FbSqlHelperParameterStructure

        
            Try 
                loFbSqlCommand = New FbCommand(psAction, poFbSqlConnection)
                loFbSqlCommand.Transaction = poFbSqlTransaction
                loFbSqlCommand.CommandTimeout = 60


                ' check for parameterized inline SQL or stored-procedure
                If ((psAction.IndexOf("@") > -1) Or _
                    (psAction.ToUpper().IndexOf("INSERT ") > -1) Or _
                    (psAction.ToUpper().IndexOf("UPDATE ") > -1) Or _
                    (psAction.ToUpper().IndexOf("DELETE ") > -1)) Then
                    loFbSqlCommand.CommandType = CommandType.Text
                Else
                    loFbSqlCommand.CommandType = CommandType.StoredProcedure      
                End If


                ' create parameters for stored-procedure
                If (Not poParameterStructures Is Nothing) Then
                    For Each loFbSqlHelperParameterStructure In poParameterStructures
                        loFbSqlParameter = New FbParameter(loFbSqlHelperParameterStructure.PARAMETER_NAME, loFbSqlHelperParameterStructure.PARAMETER_FBSQLDBTYPE) 

                        loFbSqlParameter.Direction = loFbSqlHelperParameterStructure.PARAMETER_DIRECTION

                        Select Case loFbSqlParameter.Direction 
                            Case ParameterDirection.Input:
                                loFbSqlParameter.Value = loFbSqlHelperParameterStructure.PARAMETER_VALUE
                                loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE

                            Case ParameterDirection.Output, ParameterDirection.ReturnValue:
                                loFbSqlParameter.Size = loFbSqlHelperParameterStructure.PARAMETER_SIZE
                        End Select

                        loFbSqlCommand.Parameters.Add(loFbSqlParameter)
                    Next
                End If

      
                ' execute action
                liRecordsAffected = loFbSqlCommand.ExecuteNonQuery


                'update any parameter structure values from output parameters
                For Each loFbSqlHelperParameterStructure In poParameterStructures
                    If ((loFbSqlHelperParameterStructure.PARAMETER_DIRECTION = ParameterDirection.Input) Or _
                        (loFbSqlHelperParameterStructure.PARAMETER_DIRECTION = ParameterDirection.Output)) Then

                        For Each loFbSqlParameter In loFbSqlCommand.Parameters
                            If (loFbSqlParameter.ParameterName.Trim() = loFbSqlHelperParameterStructure.PARAMETER_NAME.Trim())
                                loFbSqlHelperParameterStructure.PARAMETER_VALUE = loFbSqlParameter.Value.ToString().Trim()
                            End If
                        Next
                    End If

                    If (Not loParameterStructures.Contains(loFbSqlHelperParameterStructure)) Then
                        loParameterStructures.Add(loFbSqlHelperParameterStructure)
                    End If
                Next
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            Finally
                loFbSqlCommand.Dispose()
            End Try


             poParameterStructures = loParameterStructures

            Return (liRecordsAffected)
        End Function

    #End Region


    #Region "Private Methods -  Transaction Support Methods"

        Private Function GetFbSqlConnectionObject(ByVal psConnectionString As String) As FbConnection

            Dim loFbSqlConnection  As FbConnection = Nothing


            loFbSqlConnection = Get_DBConnection(psConnectionString)

            Return (loFbSqlConnection)      
        End Function


        Private Function GetFbSqlTransactionObject(ByVal poFbSqlConnection As FbConnection) As FbTransaction

            Dim loFbSqlTransaction  As FbTransaction


            loFbSqlTransaction = poFbSqlConnection.BeginTransaction()

            Return (loFbSqlTransaction)      
        End Function


        Private Function Execute_TransactionRollback(ByRef poFbSqlConnection As FbConnection, _
                                                     ByRef poFbSqlTransaction As FbTransaction) As Integer

            Try
                Rollback_FbSqlTransaction(poFbSqlTransaction)

				poFbSqlConnection.Close()
                poFbSqlConnection.Dispose()
                poFbSqlTransaction.Dispose()
			Catch loException As Exception
                Throw loException
			End Try  

            Return (0)
        End Function


        Private Sub Commit_FbSqlTransaction(ByVal poFbSqlTransaction  As FbTransaction)

            Try
                poFbSqlTransaction.Commit()
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            End Try
        End Sub


        Private Sub Rollback_FbSqlTransaction(ByVal poFbSqlTransaction  As FbTransaction)

            Try
                poFbSqlTransaction.Rollback()
            Catch loFbSqlException As FbException
                Throw loFbSqlException
            Catch loException As Exception
                Throw loException
            End Try
        End Sub

    #End Region


    #Region "Private Methods -  Support Methods"
    
        Private Function Get_DBConnection(ByVal psConnectionString As String) As FbConnection

            Dim loFbSqlHelperCommon  As New FbSqlHelperCommon()


            Return (loFbSqlHelperCommon.Get_DBConnection(psConnectionString))
        End Function

    #End Region


End Class
